// Unpack PreComps
// Jesse Toula

app.beginUndoGroup("Unpack Pre-Comp"); // begin the undo group so the script can be undone with one undo

var activeComp = app.project.activeItem; // save the active composition in a variable
var theLayersArray = app.project.activeItem.selectedLayers;  // put the selected layers into an array 
var originalLayers = theLayersArray; // copy the layers array in to another variable for later use

theLayersArray.sort(smallerIndex); // sort the layers array using the function defined below

function smallerIndex(a, b) {
	// compare a and b. If a is greater then b, return 1. 
	// this will sort the layers by index. That way even if the layers were selected out of order, the will now be in order
	if (a.index > b.index) {
		return 1;
	}
}

var start = theLayersArray[0].index; // get the first item's index and put it in a variable
var end = theLayersArray[theLayersArray.length - 1].index; //get the last items index and put it in a variable
var newArray = new Array; // create a new, empty array
var counter = 0; // set a counter variable to zero

// the following for loop adds all the inbetween layers into the mix, even if they were not selected.
// this was to solve a problem that when only selected layers were used, the unpacking sometimes would
// order the layers incorrectly.
for (var i = start; i <= end; i++) {
		newArray[counter] = activeComp.layer(i);
		counter++;
}

theLayersArray = newArray; // the layers array now contains all the layers between the top most and bottom most index of the selected layers
var numLayers = theLayersArray.length; // get the length of the new array


// ask the user if they would like to add the name of the precomp they came from into the layers
var keepNames = confirm("Do you want to add the pre-comp name to the extracted layers?"); 

// run the unpacking process for all the layers in the array
for (var i = 0; i < numLayers; i++) {
	
	var theLayer = theLayersArray[i]; // set theLayer to the 'i'th layer in the array
	safeToRun = true; // make safe to run true
	
	// comps that are nested are not recognized as comps. They are just seen as layers.
	// In order to find the actual composition item, we must search all of out items in the project panel
	// for a comp of the same name as the layer.
	for (var j = 1; app.project.item(j).name != theLayer.name; j++) {
		var theCompIndex = j + 1 
	}
	
	var theComp = app.project.item(theCompIndex); // when the item is found, save it in a variable
	var theMax = theComp.numLayers; //finds the number of layers in the comp

	// if the item that was found was not a composition, but a solid or footage or something, the number of layers would 
	// return null, because it does not have any layers. So we want to stop any unpacking from happening
	if (theMax == null ) {
		safeToRun = false; // if its not a comp item, make safe to run comp
		if (numLayers != 1) {
			theLayer.copyToComp(activeComp); //copys the current layer in to the  same comp to maintain the layer order
			theLayer.remove(); // deletes the duplicate that was created in the previous line
		} else {
			// if there is only one layer selected and it is not a comp item, let the user know that the script is meant for items
			alert("Nothing happened. At least one composition layer must be selected."); 
			break;
		}
	}

if (safeToRun) {
	var origLen = originalLayers.length; // find the number of originally selected layers
	var isOrig = false // make a variable that is false by default
	for (z = 0; z < origLen; z++) {
		// we are looping through all of the original layers. If the current layer was not selected by the user, we don't want to do anything to
		// it, so we are creating an intermediate variable that will be used in the next step to set the safe to run variable.
		// if the layer is found in the original layers, the isOrig layer is set to true and we break out of the for loop
		if (theLayer == originalLayers[z]) {
			isOrig = true; 
			break;
		}
	}

	// if the isOrig variable is not true, set the safe to run to false, compy the laer to maintain
	// it's position and remove the duplicate.
	if (!isOrig) {
		safeToRun = false; 
		theLayer.copyToComp(activeComp); //copys the current layer in to the order of the layers stay the same
		theLayer.remove();
	}

	// this is where the actual unpacking takes place. If safe to run is still true, the prrocess will be run
	if (safeToRun) {
		// for each layer in the current item, run the move
		for (var k = 1; k-1 < theMax; k++) {
			var moveLayer = theComp.layer(k)
			if(keepNames) { // if the user wanted to add the precomp names to the layers, the keep name variable is true
				moveLayer.name = theComp.name + "_" + moveLayer.name; // the new name is the comp_layer
			}
			moveLayer.copyToComp(activeComp); // copy the layer to the main comp
		}
		theLayer.remove(); // remove the precomp from the 
	}
}
}

app.endUndoGroup(); // end the undo group